package com.molino.mills.gui;

import java.awt.Component;
import java.awt.Container;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

/**
 * @author  Emilio
 */
public abstract class DragListener
    implements MouseMotionListener, MouseListener
{
    /**
	 * @uml.property  name="component"
	 */
    private Component component;
    /**
	 * @uml.property  name="originalContainer"
	 */
    private Container originalContainer;
    /**
	 * @uml.property  name="originalLocation"
	 */
    private Point originalLocation;
    /**
	 * @uml.property  name="dragContainer"
	 */
    private Container dragContainer;

    public synchronized void mouseDragged(MouseEvent e)
    {
        Object src = e.getSource();
        
        if ( component==null && src instanceof Component )
        {        
            Component c = (Component) src;
            
            if ( isDraggable(c) )
            {
                component = c;
                if ( dragContainer!=null )
                {                    
                    originalContainer = c.getParent();
                    originalLocation = c.getLocation();
                    originalContainer.remove(c);              
                    dragContainer.add(c);
                    dragContainer.setComponentZOrder(c, 0);
                    originalContainer.repaint();
                }
            }
        }
        
        if ( component !=null )
        {
            dragTo(e.getPoint());
        }
    }
    
    protected boolean isDraggable(Component c)
    {
        return true;
    }

    public void mouseMoved(MouseEvent e)
    {
    }

    public void mouseClicked(MouseEvent e)
    {
    }

    public void mousePressed(MouseEvent e)
    {
    }

    public void mouseReleased(MouseEvent e)
    {
        if ( component != null && e.getSource()==component )
        {
            dragReleased(e);
            component = null;
        }
    }

    protected void dragReleased(MouseEvent e)
    {
        restoreOriginalPosition();
    }

    protected void restoreOriginalPosition()
    {
        if ( dragContainer!=null )
        {
            dragContainer.remove(component);
            dragContainer.repaint();
            originalContainer.add(component);
            originalContainer.setComponentZOrder(component, 0);
        }
        component.setLocation(originalLocation);        
        component.repaint();
    }

    public void mouseEntered(MouseEvent e)
    {
    }
    
    public void mouseExited(MouseEvent e)
    {
    }

    /**
	 * @return
	 * @uml.property  name="dragContainer"
	 */
    public Container getDragContainer()
    {
        return dragContainer;
    }

    /**
	 * @param topLevelContainer
	 * @uml.property  name="dragContainer"
	 */
    public void setDragContainer(Container topLevelContainer)
    {
        this.dragContainer = topLevelContainer;
    }

    protected void dragTo(int xOffset, int yOffset)
    {       
        int xDragPoint = component.getWidth()/2;
        int yDragPoint = component.getHeight()/2;

        component.setLocation(component.getX()+xOffset-xDragPoint, component.getY()+yOffset-yDragPoint);

        component.repaint();        
    }
    
    protected void dragTo(Point dragOffset)
    {
        dragTo((int) dragOffset.getX(), (int)dragOffset.getY());
    }

    /**
	 * @return
	 * @uml.property  name="component"
	 */
    public Component getComponent()
    {
        return component;
    }

    /**
	 * @return
	 * @uml.property  name="originalContainer"
	 */
    public Container getOriginalContainer()
    {
        return originalContainer;
    }

    /**
	 * @return
	 * @uml.property  name="originalLocation"
	 */
    public Point getOriginalLocation()
    {
        return originalLocation;
    }
}
