
package com.molino.mills.model;


public class BitBoardUtil
{
    public static int positionMask(int position)
    {
        return 1 << position;
    }

    public static boolean isSet(int bb, int position)
    {
        return (bb & BitBoardUtil.positionMask(position))>0;
    }

    public static int set(int bb, int position)
    {
        return bb | BitBoardUtil.positionMask(position);
    }
    
    public static int applyMove(int bb, int moveMask)
    {
        return bb ^ moveMask;        
    }

    public static int toggle(int bb, int position)
    {
        return applyMove(bb, positionMask(position));
    }

    public static int[] positionsArrayToBitBoards(int[][] src)
    {
        int[] dest = new int[src.length];
        
        for (int i = 0; i < src.length; i++)
            dest[i] = BitBoardUtil.positionsToBitBoard(src[i]);
        
        return dest;
    }
        
    public static int positionsToBitBoard(int[] ps)
    {
        int bb = 0;
        for (int j = 0; j < ps.length; j++)
        {                
            bb = BitBoardUtil.set(bb, ps[j]);
        }
        return bb;
    }

    public static int freeSpaces(int[] bbs)
    {
        return ~(bbs[0] | bbs[1]) & 0xFFFFFF;
    }

    public static int rotateClockwise(int bb)
    {
        return ((bb & 0x3F3F3F) << 2) | ((bb & 0xC0C0C0) >>> 6);
    }

    public static int insideOut(int bb)
    {
        return ((bb & 0xFF0000) >>> 16) | (bb & 0x00FF00) | ((bb & 0x0000FF) << 16);
    }

    public static int vflip(int bb)
    {
       bb = ((bb & 0x010101) << 7) |
            ((bb & 0x020202) << 5) |
            ((bb & 0x040404) << 3) |
            ((bb & 0x080808) << 1) |
            ((bb & 0x101010) >> 1) |
            ((bb & 0x202020) >> 3) |
            ((bb & 0x404040) >> 5) |
            ((bb & 0x808080) >> 7);
        
        return ((bb & 0x010101) << 7) | ((bb & 0xFEFEFE) >>> 1); 
        
    }
}
