package com.molino.mills.model;

import com.molino.mills.model.pos.ColRowBoardPosition;

public class Board implements Cloneable
{
  
    public int[] bbs;
  
    public int[] unplacedPieces;
  
    public int[] piecesLeft;
   
    public Board()
    {
        bbs = new int[] { 0, 0 };
        unplacedPieces = new int[] { 9, 9 };
        piecesLeft = new int[] { 9, 9 };
    }

    public Board(int[] bbs, int[] unplacedPieces, int[] piecesLeft)
    {
        this.bbs = bbs;
        this.unplacedPieces = unplacedPieces;
        this.piecesLeft = piecesLeft;
    }
        
    public boolean isPieceAt(int pos, int player)
    {
        return BitBoardUtil.isSet(bbs[player], pos);
    }

    @Override
    public Board clone()
    {
        return new Board(bbs, unplacedPieces.clone(), piecesLeft.clone());
    }
    
    @Override
    public String toString()
    {
        StringBuffer sb0 = new StringBuffer();
        ColRowBoardPosition pos = new ColRowBoardPosition();
        for (int i = 0; i < 23; i++)
        {
            if ( isPieceAt(i, 0) )
            {
                if ( sb0.length() > 0 )
                    sb0.append(",");
                
                pos.setPosition(i);
                sb0.append(pos);
            }
        }
        sb0.insert(0, "Board 0: [");
        sb0.append("] ");
        
        StringBuffer sb1 = new StringBuffer();
        for (int i = 0; i < 23; i++)
        {
            if ( isPieceAt(i, 1) )
            {
                if ( sb1.length() > 0 )
                    sb1.append(",");
                pos.setPosition(i);
                sb1.append(pos);
            }
        }
        sb1.insert(0, "Board 1: [");
        sb1.append("] ");
        sb0.append(sb1);
        sb0.append(" Unplaced: ");
        sb0.append(unplacedPieces[0]);
        sb0.append("/");
        sb0.append(unplacedPieces[1]);
        
        return sb0.toString();
    }

    public int hashCode()
    {
        long x = calculateHashCode();


        return (int) ( x ^ (x>>>32) );
    }
    @Override
    public boolean equals(Object obj)
    {        
        Board other = (Board) obj;        
        
        if ( this.unplacedPieces[0] != other.unplacedPieces[0] ||
             this.unplacedPieces[1] != other.unplacedPieces[1] ||
             this.piecesLeft[0] != other.piecesLeft[0] ||
             this.piecesLeft[1] != other.piecesLeft[1] )
            return false;
        
        int this0 = this.bbs[0];
        int this1 = this.bbs[1];
        int other0 = other.bbs[0];
        int other1 = other.bbs[1];
        
        
        int bx0 = other0;
        int bx1 = other1;


        if ( this0 == bx0 && this1 == bx1 ) 
            return true;

        if ( this0 == BitBoardUtil.vflip(bx0) &&
             this1 == BitBoardUtil.vflip(bx1) )
            return true;
        

        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);
        if ( this0 == bx0 && this1 == bx1 )    
            return true;

        if ( this0 == BitBoardUtil.vflip(bx0) &&
             this1 == BitBoardUtil.vflip(bx1) )
            return true;
        

        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);
        if ( this0 == bx0 && this1 == bx1 )    
            return true;

        if ( this0 == BitBoardUtil.vflip(bx0) &&
             this1 == BitBoardUtil.vflip(bx1) )
            return true;
        

        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);
        if ( this0 == bx0 && this1 == bx1 )    
            return true;

        if ( this0 == BitBoardUtil.vflip(bx0) &&
             this1 == BitBoardUtil.vflip(bx1) )
            return true;
        

        bx0 = BitBoardUtil.insideOut(other0);
        bx1 = BitBoardUtil.insideOut(other1);
        

        if ( this0 == bx0 && this1 == bx1 ) 
            return true;

        if ( this0 == BitBoardUtil.vflip(bx0) &&
             this1 == BitBoardUtil.vflip(bx1) )
            return true;
        

        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);
        if ( this0 == bx0 && this1 == bx1 )    
            return true;

        if ( this0 == BitBoardUtil.vflip(bx0) &&
             this1 == BitBoardUtil.vflip(bx1) )
            return true;
        

        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);
        if ( this0 == bx0 && this1 == bx1 )    
            return true;

        if ( this0 == BitBoardUtil.vflip(bx0) &&
             this1 == BitBoardUtil.vflip(bx1) )
            return true;
        

        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);
        if ( this0 == bx0 && this1 == bx1 )    
            return true;

        if ( this0 == BitBoardUtil.vflip(bx0) &&
             this1 == BitBoardUtil.vflip(bx1) )
            return true;
       
        return false;
    }
    
    public long calculateHashCode()
    {        
 
        int bb0 = bbs[0];
        int bb1 = bbs[1];
        

        long x = (bb1 << 24) | bb0;

        int bx0 = bb0;
        int bx1 = bb1;
        int br0, br1;
        

        br0 = BitBoardUtil.vflip(bx0);
        br1 = BitBoardUtil.vflip(bx1);
        x = Math.min(x, (br1 << 24) | br0);          
        

        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);                  
        x = Math.min(x, (bx1 << 24) | bx0);  
        

        br0 = BitBoardUtil.vflip(bx0);
        br1 = BitBoardUtil.vflip(bx1);
        x = Math.min(x, (br1 << 24) | br0);
        

        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);
        x = Math.min(x, (bx1 << 24) | bx0);  


        br0 = BitBoardUtil.vflip(bx0);
        br1 = BitBoardUtil.vflip(bx1);
        x = Math.min(x, (br1 << 24) | br0);  
        

        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);
        x = Math.min(x, (bx1 << 24) | bx0);  


        br0 = BitBoardUtil.vflip(bx0);
        br1 = BitBoardUtil.vflip(bx1);
        x = Math.min(x, (br1 << 24) | br0);     
                
         
        bx0 = BitBoardUtil.insideOut(bb0);
        bx1 = BitBoardUtil.insideOut(bb1);
        

        x = Math.min(x, (bx1 << 24) | bx0);  
        
  
        br0 = BitBoardUtil.vflip(bx0);
        br1 = BitBoardUtil.vflip(bx1);
        x = Math.min(x, (br1 << 24) | br0);  
    
 
        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);
        x = Math.min(x, (bx1 << 24) | bx0);

 
        br0 = BitBoardUtil.vflip(bx0);
        br1 = BitBoardUtil.vflip(bx1);
        x = Math.min(x, (br1 << 24) | br0);  
        
 
        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);
        x = Math.min(x, (bx1 << 24) | bx0);  

 
        br0 = BitBoardUtil.vflip(bx0);
        br1 = BitBoardUtil.vflip(bx1);
        x = Math.min(x, (br1 << 24) | br0);  
        
 
        bx0 = BitBoardUtil.rotateClockwise(bx0);
        bx1 = BitBoardUtil.rotateClockwise(bx1);
        x = Math.min(x, (bx1 << 24) | bx0);  

 
        br0 = BitBoardUtil.vflip(bx0);
        br1 = BitBoardUtil.vflip(bx1);
        x = Math.min(x, (br1 << 24) | br0);  

        x = (x << 4) | piecesLeft[0];
        x = (x << 4) | piecesLeft[1];
        
        return x;
    }    
}
