package com.molino.mills.model;

import java.awt.Color;

import com.molino.mills.model.move.Move;

public abstract class Player
    extends Thread
{
    /**
	 * @uml.property  name="colorName"
	 */
    private String colorName;
    /**
	 * @uml.property  name="color"
	 */
    private Color color;
    /**
	 * @uml.property  name="game"
	 * @uml.associationEnd  
	 */
    private Game game;
    /**
	 * @uml.property  name="playerNumber"
	 */
    private int playerNumber;
    /**
	 * @uml.property  name="playerName"
	 */
    private String playerName;
    /**
	 * @uml.property  name="interruptRequested"
	 */
    private boolean interruptRequested = false;
    
    public Player()
    {
    }

    /**
	 * @return
	 * @uml.property  name="playerName"
	 */
    public String getPlayerName()
    {
        return playerName;
    }

    /**
	 * @param name
	 * @uml.property  name="playerName"
	 */
    public void setPlayerName(String name)
    {
        this.playerName = name;
    }

    public abstract Move getNextMove(MoveHashMap legalMoves);
    public abstract void prepareNextMove(MoveHashMap legalMoves);

    /**
	 * @return
	 * @uml.property  name="game"
	 */
    public Game getGame()
    {
        return game;
    }
    
    /**
	 * @param game
	 * @uml.property  name="game"
	 */
    public void setGame(Game game)
    {
        this.game = game;
        interruptRequested = false;
    }

    /**
	 * @return
	 * @uml.property  name="playerNumber"
	 */
    public int getPlayerNumber()
    {
        return playerNumber;
    }

    /**
	 * @param playerNumber
	 * @uml.property  name="playerNumber"
	 */
    public void setPlayerNumber(int playerNumber)
    {
        this.playerNumber = playerNumber;
    }
    
    public String toString()
    {
        return
            playerName!=null ? playerName : colorName;
    }

    public void reset()
    {
    }
    
    public synchronized void interrupt()
    {
        interruptRequested = true;
        notifyAll();
    }
    
    /**
	 * @return
	 * @uml.property  name="interruptRequested"
	 */
    public boolean isInterruptRequested()
    {
        return interruptRequested;
    }

    public void clearInterruptRequest()
    {
        interruptRequested = false;
    }
    
    public boolean isMyTurn()
    {
        return game.state.currentPlayer==playerNumber;
    }
    
    /**
	 * @return
	 * @uml.property  name="color"
	 */
    public Color getColor()
    {
        return color;
    }

    /**
	 * @param color
	 * @uml.property  name="color"
	 */
    public void setColor(Color color)
    {
        this.color = color;
    }

    /**
	 * @return
	 * @uml.property  name="colorName"
	 */
    public String getColorName()
    {
        return colorName;
    }

    /**
	 * @param colorName
	 * @uml.property  name="colorName"
	 */
    public void setColorName(String colorName)
    {
        this.colorName = colorName;
    }
}
