package com.molino.mills.model.pos;

import java.util.Arrays;
import java.util.List;


/**
 * @author  Emilio
 */
public class XYBoardPosition implements BoardPosition
{     
   
    /**
	 * @uml.property  name="pOSITIONS"
	 * @uml.associationEnd  multiplicity="(0 -1)"
	 */
    static final XYBoardPosition[] POSITIONS = {
        // Outer square
        new XYBoardPosition(-3,-3), 
        new XYBoardPosition(0,-3), 
        new XYBoardPosition(3,-3),
        new XYBoardPosition(3,0),
        new XYBoardPosition(3,3), 
        new XYBoardPosition(0,3), 
        new XYBoardPosition(-3,3),
        new XYBoardPosition(-3,0),
        // Middle square
        new XYBoardPosition(-2,-2), 
        new XYBoardPosition(0,-2), 
        new XYBoardPosition(2,-2),
        new XYBoardPosition(2,0),
        new XYBoardPosition(2,2), 
        new XYBoardPosition(0,2), 
        new XYBoardPosition(-2,2),
        new XYBoardPosition(-2,0),
        // Inner square
        new XYBoardPosition(-1,-1), 
        new XYBoardPosition(0,-1), 
        new XYBoardPosition(1,-1),
        new XYBoardPosition(1,0),
        new XYBoardPosition(1,1), 
        new XYBoardPosition(0,1), 
        new XYBoardPosition(-1,1),
        new XYBoardPosition(-1,0)
    };
    static final List<XYBoardPosition> POSITION_LIST =  Arrays.asList(POSITIONS); 
    private int xPos;
    private int yPos;
    
    public XYBoardPosition()
    {        
    }
    
    public XYBoardPosition(int position)
    {
        setPosition(position);
    }

    public XYBoardPosition(int xPos, int yPos)
    {        
        this.xPos = xPos;
        this.yPos = yPos;
    }
    
    public int getPosition()
    {
        return POSITION_LIST.indexOf(this);
    }

    public void setPosition(int position)
    {
        if ( position < 0 || position > 23 )
            throw new IllegalArgumentException("Bad position: "+position);
        
        XYBoardPosition pos = POSITION_LIST.get(position);
        
        setXYPos(pos.getXPos(), pos.getYPos());
    }

    public boolean isValid()
    {
     
        if ( xPos < -3 || xPos > 3 || yPos < -3 || yPos > 3 )
            return false;
        else
            return POSITION_LIST.contains(this);

    }
    
    public void validate() throws InvalidBoardPositionException
    {
        if ( !isValid() )
            throw new InvalidBoardPositionException("("+xPos+","+yPos+")");
    }
    
    public void setXYPos(int xPos, int yPos)
    {
        this.xPos = xPos;
        this.yPos = yPos;
    }
    
    public void setXPos(int pos)
    {
        xPos = pos;
    }

    public void setYPos(int pos)
    {
        yPos = pos;
    }

    public int getXPos()
    {
        return xPos;
    }

    public int getYPos()
    {
        return yPos;
    }

    @Override
    public int hashCode()
    {
        return (yPos+3) | ((xPos+3) << 4);
    }

    @Override
    public boolean equals(Object o)
    {
        XYBoardPosition other = (XYBoardPosition) o;
        return this.xPos==other.xPos && this.yPos==other.yPos;
    }
    @Override
    public String toString()
    {
        return "("+xPos+","+yPos+")";
    }

    public ColRowBoardPosition toColRowBoardPosition()
    {
        return new ColRowBoardPosition((char) (xPos+3+'A'), (char) (yPos+3+'1'));
    }
}
